@@ -36,7 +36,7 @@ Created 10/21/1995 Heikki Tuuri
 #ifndef os0file_h
 #define os0file_h
 
-#include "page0size.h"
+#include "fsp0types.h"
 #include "os0api.h"
 
 #ifndef _WIN32
@@ -45,12 +45,13 @@ Created 10/21/1995 Heikki Tuuri
 #include <time.h>
 #endif /* !_WIN32 */
 
+#include "my_counter.h"
+
 /** File node of a tablespace or the log data space */
 struct fil_node_t;
 struct fil_space_t;
 
 extern bool	os_has_said_disk_full;
-extern my_bool	srv_use_trim;
 
 /** File offset in bytes */
 typedef ib_uint64_t os_offset_t;
@@ -69,10 +70,6 @@ the OS actually supports it: Win 95 does not, NT does. */
 /** File handle */
 typedef HANDLE os_file_t;
 
-/** Convert a C file descriptor to a native file handle
-@param fd file descriptor
-@return native file handle */
-# define OS_FILE_FROM_FD(fd) (HANDLE) _get_osfhandle(fd)
 
 #else /* _WIN32 */
 
@@ -81,14 +78,9 @@ typedef DIR*	os_file_dir_t;	/*!< directory stream */
 /** File handle */
 typedef int	os_file_t;
 
-/** Convert a C file descriptor to a native file handle
-@param fd file descriptor
-@return native file handle */
-# define OS_FILE_FROM_FD(fd) fd
-
 #endif /* _WIN32 */
 
-static const os_file_t OS_FILE_CLOSED = os_file_t(~0);
+static const os_file_t OS_FILE_CLOSED = IF_WIN(os_file_t(INVALID_HANDLE_VALUE),-1);
 
 /** File descriptor with optional PERFORMANCE_SCHEMA instrumentation */
 struct pfs_os_file_t
@@ -252,7 +244,7 @@ public:
 		m_fil_node(NULL),
 		m_type(static_cast<uint16_t>(type))
 	{
-		if (!is_punch_hole_supported() || !srv_use_trim) {
+		if (!is_punch_hole_supported()) {
 			clear_punch_hole();
 		}
 	}
@@ -271,7 +263,7 @@ public:
 			set_punch_hole();
 		}
 
-		if (!is_punch_hole_supported() || !srv_use_trim) {
+		if (!is_punch_hole_supported()) {
 			clear_punch_hole();
 		}
 	}
@@ -358,7 +350,7 @@ public:
 	/** Set the punch hole flag */
 	void set_punch_hole()
 	{
-		if (is_punch_hole_supported() && srv_use_trim) {
+		if (is_punch_hole_supported()) {
 			m_type |= PUNCH_HOLE;
 		}
 	}
@@ -371,18 +363,8 @@ public:
 
 	/** Set the pointer to file node for IO
 	@param[in] node			File node */
-	void set_fil_node(fil_node_t* node)
-	{
-		if (!srv_use_trim ||
-		   (node && !fil_node_should_punch_hole(node))) {
-			clear_punch_hole();
-		}
+	inline void set_fil_node(fil_node_t* node);
 
-		m_fil_node = node;
-	}
-
-	/** Compare two requests
-	@reutrn true if the are equal */
 	bool operator==(const IORequest& rhs) const
 	{
 		return(m_type == rhs.m_type);
@@ -426,17 +408,7 @@ public:
 			: 0);
 	}
 
-	bool should_punch_hole() const {
-		return (m_fil_node ?
-			fil_node_should_punch_hole(m_fil_node)
-			: false);
-	}
-
-	void space_no_punch_hole() const {
-		if (m_fil_node) {
-			fil_space_set_punch_hole(m_fil_node, false);
-		}
-	}
+	inline bool should_punch_hole() const;
 
 	/** Free storage space associated with a section of the file.
 	@param[in]	fh		Open file handle
@@ -490,7 +462,7 @@ or write, causing a bottleneck for parallelism. */
 static const ulint OS_AIO_SYNC = 24;
 /* @} */
 
-extern ulint	os_n_file_reads;
+extern Atomic_counter<ulint> os_n_file_reads;
 extern ulint	os_n_file_writes;
 extern ulint	os_n_fsyncs;
 
@@ -538,14 +510,11 @@ struct os_file_stat_t {
 };
 
 /** Create a temporary file. This function is like tmpfile(3), but
-the temporary file is created in the given parameter path. If the path
-is null then it will create the file in the mysql server configuration
+the temporary file is created in the in the mysql server configuration
 parameter (--tmpdir).
-@param[in]	path	location for creating temporary file
 @return temporary file handle, or NULL on error */
 FILE*
-os_file_create_tmpfile(
-	const char*	path);
+os_file_create_tmpfile();
 
 /** The os_file_opendir() function opens a directory stream corresponding to the
 directory named by the dirname argument. The directory stream is positioned
@@ -811,9 +780,7 @@ os_file_rename
 os_aio
 os_file_read
 os_file_read_no_error_handling
-os_file_read_no_error_handling_int_fd
 os_file_write
-os_file_write_int_fd
 
 The wrapper functions have the prefix of "innodb_". */
 
@@ -849,18 +816,10 @@ The wrapper functions have the prefix of "innodb_". */
 	pfs_os_file_read_no_error_handling_func(			\
 		type, file, buf, offset, n, o, __FILE__, __LINE__)
 
-# define os_file_read_no_error_handling_int_fd(type, file, buf, offset, n) \
-	pfs_os_file_read_no_error_handling_int_fd_func(			\
-		type, file, buf, offset, n, __FILE__, __LINE__)
-
 # define os_file_write(type, name, file, buf, offset, n)	\
 	pfs_os_file_write_func(type, name, file, buf, offset,	\
 			       n, __FILE__, __LINE__)
 
-# define os_file_write_int_fd(type, name, file, buf, offset, n)		\
-	pfs_os_file_write_int_fd_func(type, name, file, buf, offset,	\
-		n, __FILE__, __LINE__)
-
 # define os_file_flush(file)					\
 	pfs_os_file_flush_func(file, __FILE__, __LINE__)
 
@@ -1197,13 +1156,9 @@ to original un-instrumented file I/O APIs */
 
 # define os_file_read_no_error_handling(type, file, buf, offset, n, o)	\
 	os_file_read_no_error_handling_func(type, file, buf, offset, n, o)
-# define os_file_read_no_error_handling_int_fd(type, file, buf, offset, n) \
-	os_file_read_no_error_handling_func(type, OS_FILE_FROM_FD(file), buf, offset, n, NULL)
 
 # define os_file_write(type, name, file, buf, offset, n)	\
 	os_file_write_func(type, name, file, buf, offset, n)
-# define os_file_write_int_fd(type, name, file, buf, offset, n)	\
-	os_file_write_func(type, name, OS_FILE_FROM_FD(file), buf, offset, n)
 
 # define os_file_flush(file)	os_file_flush_func(file)
 
@@ -1571,7 +1526,7 @@ path. If the path is NULL then it will be created on --tmpdir location.
 This function is defined in ha_innodb.cc.
 @param[in]	path	location for creating temporary file
 @return temporary file descriptor, or < 0 on error */
-int
+os_file_t
 innobase_mysql_tmpfile(
 	const char*	path);
 
@@ -1614,19 +1569,6 @@ os_file_change_size_win32(
 
 #endif /*_WIN32 */
 
-/** Check if the file system supports sparse files.
-
-Warning: On POSIX systems we try and punch a hole from offset 0 to
-the system configured page size. This should only be called on an empty
-file.
-
-@param[in]	fh		File handle for the file - if opened
-@return true if the file system supports sparse files */
-bool
-os_is_sparse_file_supported(
-	os_file_t	fh)
-	MY_ATTRIBUTE((warn_unused_result));
-
 /** Free storage space associated with a section of the file.
 @param[in]	fh		Open file handle
 @param[in]	off		Starting offset (SEEK_SET)
@@ -1666,16 +1608,6 @@ is_absolute_path(
 	return(false);
 }
 
-/***********************************************************************//**
-Try to get number of bytes per sector from file system.
-@return	file block size */
-UNIV_INTERN
-ulint
-os_file_get_block_size(
-/*===================*/
-	os_file_t	file,	/*!< in: handle to a file */
-	const char*	name);	/*!< in: file name */
-
 #include "os0file.ic"
 
 #endif /* os0file_h */
